# Memory Benchmark

The `MemoryBenchmark.ino` was compiled with each `FEATURE_*` and the flash
memory and static RAM sizes were recorded. The `FEATURE_BASELINE` selection is
the baseline, and its memory usage  numbers are subtracted from the subsequent
`FEATURE_*` memory usage.

**Version**: AceTime v1.11.5

**DO NOT EDIT**: This file was auto-generated using `make README.md`.

## How to Regenerate

To regenerate this README.md:

```
$ make clean_benchmarks
$ make benchmarks
$ make README.md
```

The `make benchmarks` target uses `collect.sh` script which calls `auniter.sh`
(https://github.com/bxparks/AUniter) to invoke the Arduino IDE programmatically.
It produces a `*.txt` file with the flash and ram usage information (e.g.
`nano.txt`). It now takes about 16 minutes to generate the `*.txt` files on my
quad-core Intel Core i7-3840QM CPU @ 2.80GHz laptop.

The `make README.md` command calls the `generated_readme.py` Python script which
generates this `README.md` file. The ASCII tables below are generated by the
`generate_table.awk` script, which takes each `*.txt` file and converts it to an
ASCII table.

## Library Size Changes

**v1.3:**
* The `BasicZoneManager` and `ExtendedZoneManager` classes were unified under a
  new parent interface `ZoneManager`. This seems to have caused the flash size
  to increase by around 1200 bytes on the AVR processors (Nano, Pro Micro),
  about 500 bytes on a SAMD, about 800 bytes on a ESP8266, 100 bytes on a ESP32,
  and 1400 bytes on a Teensy 3.2. The 8-bit processors suffer the most flash
  size increase proportional to their limited 32kB limit.
* Adding the `ZoneManager` interface simplifies a lot of the complexity with
  saving and restoring time zones using the `TimeZoneData` object, and I think
  it is worth the extra cost of flash size. The mitigating factor is that
  applications targetted towards 8-bit processors will normally have fixed
  number of timezones at compile time, so they can avoid using a `ZoneManager`,
  and avoid this penalty in flash size.

**v1.4.1+:**
* Removed the `ZoneInfo::transitionBufSize` field from the `ZoneInfo` struct,
  which saves 1 byte on 8-bit processors (none on 32-bit processors due to
  4-byte alignment). We save 266 bytes for `BasicZoneManager` and 386 bytes for
  `ExtendedZoneManager` when all the zones are loaded into the zone registry.
* Incorporated zoneName compression causes flash/ram usage to increase by
  ~250/120 bytes when using only 1-2 zones, but *decreases* flash consumption by
  1200-2400 bytes when all the zones are loaded into the `ZoneManager`.

**v1.5+:**
* Changing `ZoneProcessorCache::getType()` from a `virtual` to a non-virtual
  method saves 250-350 bytes of flash memory when using a `BasicZoneManager` or
  an `ExtendedZoneManager` on an 8-bit AVR processor. Unexpectedly, the flash
  memory consumption *increases* slightly (~0-50 bytes) for some ARM processors
  and the ESP32. Since those processors have far more flash memory, this seems
  like a good tradeoff.
* Changing `BasicZoneProcessor` and `ExtendedZoneProcessor` to be subclasses of
  the templatized `BasicZoneProcessorTemplate` and
  `ExtendedZoneProcessorTemplate` classes causes reduction of flash consumption
  by 250-400 bytes for 32-bit processors. Don't know why. (Very little
  difference for 8-bit AVR).
* Adding a `BrokerFactory` layer of indirection (to support more complex
  ZoneProcessors and Brokers) causes flash memory to go up by 100-200 bytes.

**v1.6:**
* Added support for `LinkRegistry` to `BasicZoneManager` and
  `ExtendedZoneManager`. This increases the flash memory usage by 150-500 bytes
  when using one of these classes due to the code required by `LinkRegistrar`. This extra cost is incurred even if the `LinkRegistry` is set to 0 elements. Each `LinkEntry` consumes 8 bytes (2 x `uint32_t`). So a
  `zonedb::kLinkRegistry` with 183 elements uses 1464 extra bytes of flash; a
  `zonedbx::kLinkRegistry` with 207 elements uses 1656 extra bytes.

**v1.7:**
* The virtual destructor on the `Clock` base class removed. This reduced the
  flash usage by 618 bytes on AVR processors , 328 bytes on the SAMD21, but only
  50-60 bytes on other 32-bit processors.
* The various `printShortNameTo()` or `printShortTo()` methods changed to
  replace the underscore in the zone names (e.g. `Los_Angeles`) with spaces
  (e.g. `Los Angeles`) to be more human friendly. This made little difference in
  the flash memory consumption, except on the ESP32 where it increased by
  200-300 bytes.

**v1.7.2**
* The `SystemClock::clockMillis()` is now non-virtual, using compile-time
  polymorphism through C++ template, and incorporating the same techniques from
  AceRoutine v1.3. Saves about 20-40 bytes of flash.

**v1.7.5:**
* `ExtendedZoneProcessor.compareTransitionToMatch()` was modified to
  detect an exact equality between a `Transition` and its `MatchingEra` if any
  of the 3 time stamp versions ('w', 's', 'u') are equal. Adds about 120-150
  bytes of flash on 8-bit and 32-bit processors. But removing
  `originalTransitionTime` from `Transition` decreases flash usage by about 20
  bytes.
* Upgrade ESP8266 Core from 2.7.4 to 3.0.2. Flash consumption increases by
  3-5 kB across the board.
* Upgrade Teensyduino from 1.54 to 1.55. Add memory consumed by `malloc()` and
  `free()` when using classes with virtual methods into baseline
  MemoryBenchmark, reducing the actual memory usage of various features by
  ~3kB.

**v1.8.0:**
* Move Clock and SystemClock benchmarks into AceTimeClock v1.0.0.
* Extract thin links from BasicZoneManager and ExtendedZoneManager into
  new BasicLinkManager and ExtendedLinkManager classes.
    * Saves 200-500 bytes of flash for BasicZoneManager and ExtendedZoneManager.
    * Applications can decide whether to use thin links through the LinkManager
      (~2000 flash bytes for AVR) or use fat links through the
      `kZoneAndLinkRegistry` (~5000 flash bytes for AVR).
* Create various test objects as global variables instead of stack variables
  to get a more accurate measurement of their static memory consumption.

**v1.9.0:**
* Reduce flash usage of `BasicZoneManager` and `ExtendedZoneManager` by
  1100-1300 bytes on AVR processors:
    * Extract `BasicZoneProcessorCache` and `ExtendedZoneProcessorCache` out
      of `BasicZoneManager` and `ExtendedZoneManager`, making them
      non-templatized.
    * Remove all `virtual` methods from `ZoneManager`, making the ZoneManager
      hierarchy non-polymorphic.
    * Looks like I am reverting some of the changes made in v1.3 when I created
      the `ZoneManager` interface.
* Reduce flash usage of `BasicLinkManager` and `ExtendedLinkManager` by
  68 bytes on AVR processors by removing pure `virtual` methods on `LinkManager`
  base class.
* Increase flash usage by 34 bytes on AVR processors due to slight refactoring
  of `getHighWater()` with `getAllocSize()`. Only 4-8 bytes increase on 32-bit
  processors.

**v1.10.0:**
* Remove support for SAMD21 boards.
    * Arduino IDE 1.8.19 with SparkFun SAMD 1.8.6 can no longer upload binaries
      to these boards. Something about bossac 1.7.0 not found.
* Add memory consumption benchmarks for `ZoneSorterByName` and
  `ZoneSorterByOffsetAndName` for `BasicZoneManager` and `ExtendedZoneManager`.
    * AVR: 180-530 bytes of flash
    * 32-bit: 120-600 bytes of flash
* Upgrade tool chain:
    * Arduino IDE from 1.8.13 to 1.8.19
    * Arduino AVR from 1.8.3 to 1.8.4
    * STM32duino from 2.0.0 to 2.2.0
    * ESP32 from 1.0.6 to 2.0.2
    * Teensyduino from 1.55 to 1.56
* Add support for `fold` parameter in `LocalDateTime`, `OffsetDateTime`,
  `ZonedDateTime`, and `ExtendedZoneProcessor`. Increases flash usage:
    * AVR:
        * ~600 bytes, in `ExtendedZoneProcessor` for additional search logic,
        * ~150 bytes, `BasicZoneProcessor`, to carry along the `fold` parameter
    * most 32-bit: 400-600 bytes
    * Teensy: 1300 bytes (no idea why)

**v1.11.0**
* Upgrade ZoneInfo database so that Links are symbolic links to Zones, instead
  of hard links to Zones.
    * Allows Links to know whether they are links.
    * Allows extraction of the zoneId and zoneNames of the target Zone.
    * AVR: Increases flash consumption by ~270 bytes.
    * STM32: Increases flash by 120-150 bytes.
    * ESP8266: Increases flash by 250-300 bytes.
    * ESP32: Increases flash by ~190 bytes.
    * Teensy 3.2: Increase flash by 450-1300 bytes.

**v1.11.1**
* Change `ZoneInfoBroker::targetZoneInfo()` method to return a `ZoneInfoBroker`
  instead of raw `ZoneInfo*` pointer.
    * Increases flash usage by 4-16 bytes for the most part.

**v1.11.5**
* Upgrade tool chain
    * Arduino CLI from 0.20.2 to 0.27.1
    * Arduino AVR Core from 1.8.4 to 1.8.5
    * STM32duino from 2.2.0 to 2.3.0
    * ESP32 Core from 2.0.2 to 2.0.5
    * Teensyduino from 1.56 to 1.57
* Upgrade TZDB from 2022b to 2022d

# Legend

* [1] Delta flash and ram consumption for `ZoneSorterByName` and
  `ZoneSorterByOffsetAndName` are calculated by subtracting the
  `BasicZoneManager (1 zone)` numbers, to isolate the memory consumption
  of just the sorter classes.
* [2] Delta flash and ram consumption for `ZoneSorterByName` and
  `ZoneSorterByOffsetAndName` are calculated by subtracting the
  `ExtendedZoneManager (1 zone)` numbers, to isolate the memory
  consumption of just the sorter classes.

## Arduino Nano

* 16MHz ATmega328P
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* Arduino AVR Boards 1.8.5

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |    474/   11 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   1132/   20 |   658/    9 |
| ZonedDateTime                          |   1306/   27 |   832/   16 |
| Manual ZoneManager                     |   1242/   13 |   768/    2 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |   6534/  319 |  6060/  308 |
| Basic TimeZone (2 zones)               |   6714/  439 |  6240/  428 |
| BasicZoneManager (1 zone)              |   6740/  330 |  6266/  319 |
| BasicZoneManager (all zones)           |  18484/  706 | 18010/  695 |
| BasicZoneManager (all zones+links)     |  23346/  706 | 22872/  695 |
| BasicLinkManager (all links)           |   2570/   16 |  2096/    5 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             |   6904/  330 |   370/   11 |
| Basic ZoneSorterByOffsetAndName [1]    |   7028/  330 |   494/   11 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |   9842/  674 |  9368/  663 |
| Extended TimeZone (2 zones)            |  10088/ 1115 |  9614/ 1104 |
| ExtendedZoneManager (1 zone)           |  10018/  680 |  9544/  669 |
| ExtendedZoneManager (all zones)        |  30810/ 1164 | 30336/ 1153 |
| ExtendedZoneManager (all zones+links)  |  36222/ 1164 | 35748/ 1153 |
| ExtendedLinkManager (all links)        |   2762/   16 |  2288/    5 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          |  10214/  680 |   372/    6 |
| Extended ZoneSorterByOffsetAndName [2] |  10336/  680 |   494/    6 |
+---------------------------------------------------------------------+

```

## Sparkfun Pro Micro

* 16 MHz ATmega32U4
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* SparkFun AVR Boards 1.1.13

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |   3470/  153 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   4104/  160 |   634/    7 |
| ZonedDateTime                          |   4278/  167 |   808/   14 |
| Manual ZoneManager                     |   4236/  153 |   766/    0 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |   9488/  457 |  6018/  304 |
| Basic TimeZone (2 zones)               |   9670/  579 |  6200/  426 |
| BasicZoneManager (1 zone)              |   9694/  468 |  6224/  315 |
| BasicZoneManager (all zones)           |  21456/  846 | 17986/  693 |
| BasicZoneManager (all zones+links)     |  26318/  846 | 22848/  693 |
| BasicLinkManager (all links)           |   5544/  158 |  2074/    5 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             |   9860/  470 |   372/   13 |
| Basic ZoneSorterByOffsetAndName [1]    |   9984/  470 |   496/   13 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |  12796/  812 |  9326/  659 |
| Extended TimeZone (2 zones)            |  13044/ 1255 |  9574/ 1102 |
| ExtendedZoneManager (1 zone)           |  12972/  818 |  9502/  665 |
| ExtendedZoneManager (all zones)        |  33780/ 1302 | 30310/ 1149 |
| ExtendedZoneManager (all zones+links)  |  39192/ 1302 | 35722/ 1149 |
| ExtendedLinkManager (all links)        |   5736/  158 |  2266/    5 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          |  13170/  820 |   374/    8 |
| Extended ZoneSorterByOffsetAndName [2] |  13292/  820 |   496/    8 |
+---------------------------------------------------------------------+

```

## STM32 Blue Pill

* STM32F103C8, 72 MHz ARM Cortex-M3
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* STM32duino 2.3.0

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |  21336/ 3556 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  21640/ 3564 |   304/    8 |
| ZonedDateTime                          |  21872/ 3580 |   536/   24 |
| Manual ZoneManager                     |  21868/ 3556 |   532/    0 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |  25696/ 3720 |  4360/  164 |
| Basic TimeZone (2 zones)               |  25996/ 3884 |  4660/  328 |
| BasicZoneManager (1 zone)              |  25816/ 3740 |  4480/  184 |
| BasicZoneManager (all zones)           |  41316/ 3740 | 19980/  184 |
| BasicZoneManager (all zones+links)     |  48576/ 3740 | 27240/  184 |
| BasicLinkManager (all links)           |  23240/ 3564 |  1904/    8 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             |  25992/ 3744 |   296/   24 |
| Basic ZoneSorterByOffsetAndName [1]    |  26040/ 3744 |   344/   24 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |  27684/ 4108 |  6348/  552 |
| Extended TimeZone (2 zones)            |  27968/ 4660 |  6632/ 1104 |
| ExtendedZoneManager (1 zone)           |  27788/ 4116 |  6452/  560 |
| ExtendedZoneManager (all zones)        |  55676/ 4116 | 34340/  560 |
| ExtendedZoneManager (all zones+links)  |  63740/ 4116 | 42404/  560 |
| ExtendedLinkManager (all links)        |  23432/ 3564 |  2096/    8 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          |  27972/ 4120 |   288/   12 |
| Extended ZoneSorterByOffsetAndName [2] |  28028/ 4120 |   344/   12 |
+---------------------------------------------------------------------+

```

An entry of `-1` indicates that the memory usage exceeded the maximum of the
microcontroller and the compiler did not generate the desired information.

## ESP8266

* NodeMCU 1.0, 80MHz ESP8266
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* ESP8266 Boards 3.0.2

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               | 260089/27892 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 260545/27908 |   456/   16 |
| ZonedDateTime                          | 260961/27924 |   872/   32 |
| Manual ZoneManager                     | 260941/27896 |   852/    4 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                | 266777/28644 |  6688/  752 |
| Basic TimeZone (2 zones)               | 267177/28804 |  7088/  912 |
| BasicZoneManager (1 zone)              | 266937/28660 |  6848/  768 |
| BasicZoneManager (all zones)           | 282873/28660 | 22784/  768 |
| BasicZoneManager (all zones+links)     | 290409/28660 | 30320/  768 |
| BasicLinkManager (all links)           | 262125/27904 |  2036/   12 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             | 267173/28664 |   396/   20 |
| Basic ZoneSorterByOffsetAndName [1]    | 267285/28664 |   508/   20 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             | 269353/29172 |  9264/ 1280 |
| Extended TimeZone (2 zones)            | 269769/29724 |  9680/ 1832 |
| ExtendedZoneManager (1 zone)           | 269481/29180 |  9392/ 1288 |
| ExtendedZoneManager (all zones)        | 298013/29176 | 37924/ 1284 |
| ExtendedZoneManager (all zones+links)  | 306397/29176 | 46308/ 1284 |
| ExtendedLinkManager (all links)        | 262317/27904 |  2228/   12 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          | 269701/29184 |   348/   12 |
| Extended ZoneSorterByOffsetAndName [2] | 269797/29184 |   444/   12 |
+---------------------------------------------------------------------+

```

## ESP32

* ESP32-01 Dev Board, 240 MHz Tensilica LX6
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* ESP32 Boards 2.0.5

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               | 211065/16056 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 213317/16064 |  2252/    8 |
| ZonedDateTime                          | 213769/16080 |  2704/   24 |
| Manual ZoneManager                     | 213773/16056 |  2708/    0 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                | 218333/16216 |  7268/  160 |
| Basic TimeZone (2 zones)               | 218781/16384 |  7716/  328 |
| BasicZoneManager (1 zone)              | 218493/16240 |  7428/  184 |
| BasicZoneManager (all zones)           | 234413/16240 | 23348/  184 |
| BasicZoneManager (all zones+links)     | 241933/16240 | 30868/  184 |
| BasicLinkManager (all links)           | 214841/16064 |  3776/    8 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             | 218649/16240 |   316/   24 |
| Basic ZoneSorterByOffsetAndName [1]    | 218705/16240 |   372/   24 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             | 220677/16608 |  9612/  552 |
| Extended TimeZone (2 zones)            | 221113/17160 | 10048/ 1104 |
| ExtendedZoneManager (1 zone)           | 220789/16616 |  9724/  560 |
| ExtendedZoneManager (all zones)        | 249285/16616 | 38220/  560 |
| ExtendedZoneManager (all zones+links)  | 257685/16616 | 46620/  560 |
| ExtendedLinkManager (all links)        | 215033/16064 |  3968/    8 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          | 220937/16616 |   260/    8 |
| Extended ZoneSorterByOffsetAndName [2] | 221005/16616 |   328/    8 |
+---------------------------------------------------------------------+

```

RAM usage remains constant as more objects are created, which indicates that an
initial pool of a certain minimum size is created regardless of the actual RAM
usage by objects.

## Teensy 3.2

* 96 MHz ARM Cortex-M4
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* Teensyduino 1.57

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |  10060/ 4152 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  10388/ 4168 |   328/   16 |
| ZonedDateTime                          |  10476/ 4184 |   416/   32 |
| Manual ZoneManager                     |  10184/ 4160 |   124/    8 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |  21580/ 4324 | 11520/  172 |
| Basic TimeZone (2 zones)               |  22468/ 4488 | 12408/  336 |
| BasicZoneManager (1 zone)              |  21840/ 4344 | 11780/  192 |
| BasicZoneManager (all zones)           |  37788/ 4344 | 27728/  192 |
| BasicZoneManager (all zones+links)     |  45316/ 4344 | 35256/  192 |
| BasicLinkManager (all links)           |  11976/ 4160 |  1916/    8 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             |  21704/ 4340 |   124/   16 |
| Basic ZoneSorterByOffsetAndName [1]    |  22216/ 4340 |   636/   16 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |  24952/ 4712 | 14892/  560 |
| Extended TimeZone (2 zones)            |  25840/ 5264 | 15780/ 1112 |
| ExtendedZoneManager (1 zone)           |  25148/ 4720 | 15088/  568 |
| ExtendedZoneManager (all zones)        |  53672/ 4720 | 43612/  568 |
| ExtendedZoneManager (all zones+links)  |  62064/ 4720 | 52004/  568 |
| ExtendedLinkManager (all links)        |  12168/ 4160 |  2108/    8 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          |  25076/ 4716 |   124/    4 |
| Extended ZoneSorterByOffsetAndName [2] |  25524/ 4716 |   572/    4 |
+---------------------------------------------------------------------+

```

